"""
Mock Sensor module for use with the tests.
"""

from unittest.mock import Mock

from ...types import ConfigType, SensorValueType
from . import GenericSensor

REQUIREMENTS = ()
CONFIG_SCHEMA = dict(test=dict(type="boolean", required=False, default=False))


# pylint: disable=useless-super-delegation
class Sensor(GenericSensor):
    """
    Mock Sensor class for use with the tests.
    """

    def __init__(self, config: ConfigType):
        self.setup_module = Mock()  # type: ignore[assignment]
        self.setup_sensor = Mock()  # type: ignore[assignment]
        self.get_value = Mock(return_value=1)  # type: ignore[assignment]
        super().__init__(config)

    def setup_module(self) -> None:
        return super().setup_module()

    def setup_sensor(self, sens_conf: ConfigType) -> None:
        return super().setup_sensor(sens_conf)

    def get_value(self, sens_conf: ConfigType) -> SensorValueType:
        return super().get_value(sens_conf)



"""
HCSR04 ultrasonic range sensor (connected to the Raspberry Pi on-board GPIO)
"""

import time
from statistics import mean
from typing import Any, Dict, List, Optional, cast

from ...types import CerberusSchemaType, ConfigType, SensorValueType
from . import GenericSensor

REQUIREMENTS = ("RPi.GPIO",)

# HC-SR04 ultrasonic sensor_inputs
# for use at Raspberry Pi, the ECHO signal of the sensor must be reduced to 3.3V, since
# the sensor works with 5V!!
# You can simply reduce with a 3.9kOhm and 6.8kOhm resistor voltage reduction
#
# further information at http://www.netzmafia.de/skripten/hardware/RasPi/Projekt-Ultraschall/
# The code is based on the sample code of this webpage by Prof.Plate / University of Munich

# duration trigger pulse
PULSE = 0.00001
# sonic speed/2
SPEED_2 = 34300 / 2


class HCSR04:
    """
    Separate class for the distance sensors themselves, since there may be more than one
    attached to the Raspberry Pi's GPIO pins.
    """

    def __init__(
        self,
        gpio: Any,
        name: str,
        pin_echo: int,
        pin_trigger: int,
        burst: int,
        **kwargs: Any
    ):
        self.gpio = gpio
        self.name = name
        self.pin_echo = pin_echo
        self.pin_trigger = pin_trigger
        self.burst = burst
        self.start: Optional[float] = None
        self.distance: Optional[float] = None

        self.gpio.setup(self.pin_trigger, self.gpio.OUT)
        self.gpio.setup(self.pin_echo, self.gpio.IN)
        self.gpio.remove_event_detect(self.pin_echo)
        self.gpio.output(self.pin_trigger, False)

        # Setup-time for Sensor
        time.sleep(1)

        # create callback triggered by rising and falling edge
        def measure_callback(pin: int) -> None:
            if self.gpio.input(self.pin_echo) == 1:
                # Echo measuring begins
                self.start = time.time()
            elif self.start is not None:
                # Echo measuring completed
                delta = time.time() - self.start
                self.distance = delta * SPEED_2

        self.gpio.add_event_detect(
            self.pin_echo, self.gpio.BOTH, callback=measure_callback
        )

    def pulse(self) -> None:
        """
        Starts measurement
        """
        self.start = None
        self.distance = None
        # create trigger pulse
        self.gpio.output(self.pin_trigger, True)
        time.sleep(PULSE)
        self.gpio.output(self.pin_trigger, False)

    def measure_range(self) -> float:
        """
        Start measures and calculate average of BURST measurements
        """
        measurements: List[float] = []
        for _ in range(self.burst):
            self.pulse()
            timeout = time.time() + 1
            while self.distance is None:
                if time.time() > timeout:
                    break
                time.sleep(0.04)
            # There's no way that self.distance can be None now. Thanks, GIL!
            measurements.append(cast(float, self.distance))
            time.sleep(0.05)
        if not measurements:
            raise RuntimeError(
                "Unable to measure range on HC-SR04 sensor '%s'" % self.name
            )
        return mean(measurements)


class Sensor(GenericSensor):
    """
    Implementation of the sensor using Raspberry Pi on-board GPIO.
    """

    SENSOR_SCHEMA: CerberusSchemaType = {
        "pin_echo": {"type": "integer", "required": True, "empty": False},
        "pin_trigger": {"type": "integer", "required": True, "empty": False},
        "burst": {"type": "integer", "required": True, "empty": False},
    }

    def setup_module(self) -> None:
        # pylint: disable=import-outside-toplevel,import-error
        import RPi.GPIO as GPIO  # type: ignore

        # use BCM GPIO-references (instead of Pin-numbers)
        GPIO.setmode(GPIO.BCM)
        self.gpio = GPIO
        self.sensors: Dict[str, HCSR04] = {}

    def setup_sensor(self, sens_conf: ConfigType) -> None:
        sensor = HCSR04(gpio=self.gpio, **sens_conf)
        self.sensors[sensor.name] = sensor

    def get_value(self, sens_conf: ConfigType) -> SensorValueType:
        return self.sensors[sens_conf["name"]].measure_range()

    def cleanup(self) -> None:
        self.gpio.cleanup()



"""
Exceptions thrown by MQTT IO.
"""


class ConfigError(Exception):
    """
    Base class for any error raised because of a problem with the config file.
    """


class ConfigValidationFailed(ConfigError):
    """
    The config file validation failed.
    """


class RuntimeConfigError(ConfigError):
    """
    Something in the config file turned out to be unusable at runtime.
    """


class CannotInstallModuleRequirements(Exception):
    """
    Installing the Python requirements for a module using pip failed.
    """



"""
Contains stuff useful across all modules, whether they're GPIO, sensor or stream ones.
"""

import logging
import sys
from subprocess import CalledProcessError, check_call
from types import ModuleType
from typing import List

import pkg_resources

from ..exceptions import CannotInstallModuleRequirements

_LOG = logging.getLogger(__name__)


def install_missing_requirements(pkgs_required: List[str]) -> None:
    """
    Use pip to install the list of requirements.
    """
    check_call([sys.executable, "-m", "pip", "install"] + pkgs_required)


def install_missing_module_requirements(module: ModuleType) -> None:
    """
    Some of the modules require external packages to be installed. This gets
    the list from the `REQUIREMENTS` module attribute and attempts to
    install the requirements using pip.
    :param module: GPIO module
    :type module: ModuleType
    :return: None
    :rtype: NoneType
    """
    reqs = getattr(module, "REQUIREMENTS", [])
    if not reqs:
        _LOG.debug("Module %r has no extra requirements to install.", module)
        return

    pkgs_installed = pkg_resources.WorkingSet()
    pkgs_required = []
    for req in reqs:
        if pkgs_installed.find(pkg_resources.Requirement.parse(req)) is None:
            pkgs_required.append(req)

    if not pkgs_required:
        _LOG.debug("Module %r has all of its requirements installed already.", module)
        return

    try:
        install_missing_requirements(pkgs_required)
    except CalledProcessError as err:
        raise CannotInstallModuleRequirements(
            "Unable to install packages for module %r (%s): %s"
            % (module, pkgs_required, err)
        ) from err



"""
Contains the base class that is shared across all Sensor modules.
"""

import abc
import asyncio
from concurrent.futures import ThreadPoolExecutor
from typing import Any

from ...types import ConfigType, SensorValueType


class GenericSensor(abc.ABC):
    """
    Abstracts a generic sensor interface to be implemented
    by the modules in this directory.
    """

    def __init__(self, config: ConfigType):
        self.config = config
        self.sensor: Any = None
        self.setup_module()

    @abc.abstractmethod
    def get_value(self, sens_conf: ConfigType) -> SensorValueType:
        """
        Read the sensor's current value.
        """

    def setup_module(self) -> None:
        """
        Called on initialisation of the Sensor module during the startup phase.

        The module's config from the `sensor_modules` section of the config file is stored
        in `self.config`.
        """

    def setup_sensor(self, sens_conf: ConfigType) -> None:
        """
        Called on initialisation of each reading type of the Sensor module during the
        startup phase.

        The `sens_conf` passed in here is the sensor's entry in the `sensor_inputs`
        section of the config file.
        """

    def cleanup(self) -> None:
        """
        Called when closing the program to handle any cleanup operations.
        """

    async def async_get_value(self, sens_conf: ConfigType) -> SensorValueType:
        """
        Use a ThreadPoolExecutor to call the module's synchronous get_value function.
        """
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(ThreadPoolExecutor(), self.get_value, sens_conf)



"""
Implementation of AbstractMQTTClient using asyncio-mqtt client.
"""

import asyncio
import logging
from asyncio.queues import QueueFull
from functools import wraps
from typing import Any, Callable, List, Optional, Tuple, TypeVar, cast

from asyncio_mqtt.client import Client, MqttError, Will  # type: ignore
from paho.mqtt import client as paho  # type: ignore

from . import (
    AbstractMQTTClient,
    MQTTClientOptions,
    MQTTException,
    MQTTMessage,
    MQTTMessageSend,
    MQTTProtocol,
)

_LOG = logging.getLogger(__name__)

Func = TypeVar("Func", bound=Callable[..., Any])


def _map_exception(func: Func) -> Func:
    @wraps(func)
    async def inner(*args: Any, **kwargs: Any) -> Any:
        try:
            await func(*args, **kwargs)
        except MqttError as exc:
            raise MQTTException from exc

    return cast(Func, inner)


class MQTTClient(AbstractMQTTClient):
    """
    MQTTClient implementation using asyncio-mqtt client.
    """

    def __init__(self, options: MQTTClientOptions):
        super().__init__(options)
        protocol_map = {
            MQTTProtocol.V31: paho.MQTTv31,
            MQTTProtocol.V311: paho.MQTTv311,
            MQTTProtocol.V5: paho.MQTTv5,
        }
        will = None
        if options.will is not None:
            will = Will(
                topic=options.will.topic,
                payload=options.will.payload,
                qos=options.will.qos,
                retain=options.will.retain,
            )
        tls_context = None
        if options.tls_options is not None:
            tls_context = options.tls_options.ssl_context

        self._client = Client(
            hostname=options.hostname,
            port=options.port,
            username=options.username,
            password=options.password,
            client_id=options.client_id,
            # keepalive=options.keepalive,  # This isn't implemented yet on 0.8.1
            tls_context=tls_context,
            protocol=protocol_map[options.protocol],
            will=will,
            clean_session=options.clean_session,
        )
        self._message_queue: Optional[asyncio.Queue[MQTTMessage]] = None

    @_map_exception
    async def connect(self, timeout: int = 10) -> None:
        await self._client.connect(timeout=timeout)

    @_map_exception
    async def disconnect(self) -> None:
        try:
            await self._client.disconnect()
        except TimeoutError:
            await self._client.force_disconnect()

    @_map_exception
    async def subscribe(self, topics: List[Tuple[str, int]]) -> None:
        await self._client.subscribe(topics)

    @_map_exception
    async def publish(self, msg: MQTTMessageSend) -> None:
        await self._client.publish(
            topic=msg.topic, payload=msg.payload, qos=msg.qos, retain=msg.retain
        )

    def _on_message(
        self, client: paho.Client, userdata: Any, msg: paho.MQTTMessage
    ) -> None:
        if self._message_queue is None:
            _LOG.warning("Discarding MQTT message because queue is not initialised")
            return
        our_msg = MQTTMessage(topic=msg.topic, payload=msg.payload)
        try:
            self._message_queue.put_nowait(our_msg)
        except QueueFull:
            _LOG.warning("Discarding old MQTT message because queue is full")
            self._message_queue.get_nowait()
            self._message_queue.put_nowait(our_msg)

    @property
    def message_queue(self) -> "asyncio.Queue[MQTTMessage]":
        if self._message_queue is None:
            self._message_queue = asyncio.Queue(self._options.message_queue_size)
            # pylint: disable=protected-access
            self._client._client.on_message = (  # type: ignore[attr-defined]
                self._on_message
            )
        return self._message_queue



import asyncio
from typing import Any


# fixture of event handler that calls a mock, so we can see if an event has been called


def before_scenario(context: Any, scenario: Any) -> None:
    """
    Initialise data.
    """
    context.loop = asyncio.new_event_loop()
    context.data = dict(
        raw_config={},
        loop=context.loop,
        transient_tasks=[],
        event_subs={},
        mocks={},
    )


def after_scenario(context: Any, scenario: Any) -> None:
    """
    Shut down asyncio loop cleanly.
    """
    loop: asyncio.AbstractEventLoop = context.loop
    try:
        mqttio = context.data["mqttio"]
    except (KeyError, AttributeError):
        pass
    else:
        loop.run_until_complete(mqttio.shutdown())
    loop.close()



"""
GPIO module for writing outputs to the console. Used for testing.
"""

from typing import Optional

from ...types import ConfigType, PinType
from . import GenericGPIO, PinDirection, PinPUD


class GPIO(GenericGPIO):
    """
    Implementation of GPIO class for outputting to STDIO.
    """

    def setup_module(self) -> None:
        print("setup_module()")

    def setup_pin(
        self,
        pin: PinType,
        direction: PinDirection,
        pullup: PinPUD,
        pin_config: ConfigType,
        initial: Optional[str] = None,
    ) -> None:
        print(
            "setup_pin(pin=%r, direction=%r, pullup=%r, pin_config=%r, initial=%r)"
            % (pin, direction, pullup, pin_config, initial)
        )
        if initial == "high":
            self.set_pin(pin, True)
        elif initial == "low":
            self.set_pin(pin, False)

    async def async_set_pin(self, pin: PinType, value: bool) -> None:
        self.set_pin(pin, value)

    def set_pin(self, pin: PinType, value: bool) -> None:
        print("set_pin(pin=%r, value=%r)" % (pin, value))

    async def async_get_pin(self, pin: PinType) -> bool:
        return self.get_pin(pin)

    def get_pin(self, pin: PinType) -> bool:
        print("get_pin(pin=%r)" % pin)
        return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mqtt_io',
 'mqtt_io.config',
 'mqtt_io.config.validation',
 'mqtt_io.modules',
 'mqtt_io.modules.gpio',
 'mqtt_io.modules.sensor',
 'mqtt_io.modules.stream',
 'mqtt_io.mqtt',
 'mqtt_io.tests.features',
 'mqtt_io.tests.features.steps']

package_data = \
{'': ['*']}

install_requires = \
['Cerberus>=1.3.2,<2.0.0',
 'PyYAML>=5.4,<6.0',
 'asyncio-mqtt>=0.8.1,<0.9.0',
 'backoff>=1.10.0,<2.0.0',
 'typing-extensions>=3.7.4,<4.0.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0.8,<0.9']}

setup_kwargs = {
    'name': 'mqtt-io',
    'version': '2.2.1',
    'description': 'Expose GPIO modules (Raspberry Pi, Beaglebone, PCF8754, PiFace2 etc.), digital sensors (LM75 etc.) and serial streams to an MQTT server for remote control and monitoring.',
    'long_description': "<!--\n***************************************************************************************\nDO NOT EDIT README.md DIRECTLY, IT'S GENERATED FROM README.md.j2 USING generate_docs.py\n***************************************************************************************\n-->\n\n# MQTT IO\n\n_Documentation version: `develop`_\n\n[![Discord](https://img.shields.io/discord/713749043662290974.svg?label=Chat%20on%20Discord&logo=discord&logoColor=ffffff&color=7389D8&labelColor=6A7EC2)](https://discord.gg/gWyV9W4)\n\nExposes general purpose inputs and outputs (GPIO), hardware sensors and serial devices to an MQTT server. Ideal for single-board computers such as the Raspberry Pi.\n\nVisit the [documentation](https://mqtt-io.app/) for more detailed information.\n\n## Supported Hardware\n\nHardware support is provided by specific GPIO, Sensor and Stream modules. It's easy to add support for new hardware and the list is growing fast.\n\n### GPIO Modules\n\n  - Beaglebone GPIO (`beaglebone`)\n  - Linux Kernel 4.8+ libgpiod (`gpiod`)\n  - MCP23017 IO expander (`mcp23017`)\n  - Orange Pi GPIO (`orangepi`)\n  - PCF8574 IO expander (`pcf8574`)\n  - PCF8575 IO expander (`pcf8575`)\n  - PiFace Digital IO 2 (`piface2`)\n  - Raspberry Pi GPIO (`raspberrypi`)\n\n### Sensors\n\n  - ADS1x15 analog to digital converters (`ads1x15`)\n  - AHT20 temperature and humidity sensor (`aht20`)\n  - BH1750 light level sensor (`bh1750`)\n  - BME280 temperature, humidity and pressure sensor (`bme280`)\n  - BME680 temperature, humidity and pressure sensor (`bme680`)\n  - DHT11/DHT22/AM2302 temperature and humidity sensors (`dht22`)\n  - DS18S20/DS1822/DS18B20/DS1825/DS28EA00/MAX31850K temperature sensors (`ds18b`)\n  - HCSR04 ultrasonic range sensor (connected to the Raspberry Pi on-board GPIO) (`hcsr04`)\n  - LM75 temperature sensor (`lm75`)\n  - MCP3008 analog to digital converter (`mcp3008`)\n\n### Streams\n\n  - Serial port (`serial`)\n\n## Installation\n\n_Requires Python 3.6+_\n\n`pip3 install mqtt-io`\n\n## Execution\n\n`python3 -m mqtt_io config.yml`\n\n## Configuration Example\n\nConfiguration is written in a YAML file which is passed as an argument to the server on startup.\n\nSee the [full configuration documentation](https://github.com/flyte/pi-mqtt-gpio/wiki/Configuration) for details.\n\nThe following example will configure the software to do the following:\n\n- Publish MQTT messages on the `home/input/doorbell` topic when the doorbell is pushed and released.\n- Subscribe to the MQTT topic `home/output/port_light/set` and change the output when messages are received on it.\n- Periodically read the value of the LM75 sensor and publish it on the MQTT topic `home/sensor/porch_temperature`.\n- Publish any data received on the `/dev/ttyUSB0` serial port to the MQTT topic `home/serial/alarm_system`.\n- Subscribe to the MQTT topic `home/serial/alarm_system/send` and send any data received on that topic to the serial port.\n\n```yaml\nmqtt:\n  host: localhost\n  topic_prefix: home\n\n# GPIO\ngpio_modules:\n  # Use the Raspberry Pi built-in GPIO\n  - name: rpi\n    module: raspberrypi\n\ndigital_inputs:\n  # Pin 0 is an input connected to a doorbell button\n  - name: doorbell\n    module: rpi\n    pin: 0\n\ndigital_outputs:\n  # Pin 1 is an output connected to a light\n  - name: porch_light\n    module: rpi\n    pin: 1\n\n# Sensors\nsensor_modules:\n  # An LM75 sensor attached to the I2C bus\n  - name: lm75_sensor\n    module: lm75\n    i2c_bus_num: 1\n    chip_addr: 0x48\n\nsensor_inputs:\n  # The configuration of the specific sensor value to use (LM75 only has temperature)\n  - name: porch_temperature\n    module: lm75_sensor\n\n# Streams\nstream_modules:\n  # A serial port to communicate with the house alarm system\n  - name: alarm_system\n    module: serial\n    device: /dev/ttyUSB0\n    baud: 9600\n```",
    'author': 'Ellis Percival',
    'author_email': 'mqtt-io@failcode.co.uk',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""
Serial port
"""

from typing import Optional

from . import GenericStream

REQUIREMENTS = ("pyserial",)

BYTESIZE_CHOICES = (5, 6, 7, 8)
PARITY_CHOICES = ("none", "odd", "even", "mark", "space")
STOPBITS_CHOICES = (1, 1.5, 2)

CONFIG_SCHEMA = {
    "device": {"type": "string", "required": True, "empty": False},
    "baud": {"type": "integer", "required": True, "empty": False},
    "timeout": {"type": "integer", "required": False, "empty": False, "default": 20},
    "bytesize": {
        "type": "integer",
        "required": False,
        "default": 8,
        "empty": False,
        "allowed": BYTESIZE_CHOICES,
    },
    "parity": {
        "type": "string",
        "required": False,
        "default": "none",
        "empty": False,
        "allowed": PARITY_CHOICES,
    },
    "stopbits": {
        "type": "float",
        "required": False,
        "default": 1,
        "empty": False,
        "allowed": STOPBITS_CHOICES,
    },
}

# pylint: disable=no-member

# IDEA: Possible implementations -@flyte at 04/03/2021, 18:32:05
# Use the asyncio event loop to fire a callback when data is available on the serial port?


class Stream(GenericStream):
    """
    Stream module for sending to and receiving from serial ports.
    """

    def setup_module(self) -> None:
        # pylint: disable=import-error,import-outside-toplevel
        import serial  # type: ignore

        self.ser = serial.Serial(
            port=self.config["device"],
            baudrate=self.config["baud"],
            timeout=self.config["timeout"],
            bytesize={
                5: serial.FIVEBITS,
                6: serial.SIXBITS,
                7: serial.SEVENBITS,
                8: serial.EIGHTBITS,
            }[self.config["bytesize"]],
            parity=dict(
                none=serial.PARITY_NONE,
                odd=serial.PARITY_ODD,
                even=serial.PARITY_EVEN,
                mark=serial.PARITY_MARK,
                space=serial.PARITY_SPACE,
            )[self.config["parity"]],
            stopbits={
                1: serial.STOPBITS_ONE,
                1.5: serial.STOPBITS_ONE_POINT_FIVE,
                2: serial.STOPBITS_TWO,
            }[self.config["stopbits"]],
        )
        self.ser.flushInput()

    def read(self) -> Optional[bytes]:
        return self.ser.read(self.ser.in_waiting) or None

    def write(self, data: bytes) -> None:
        self.ser.write(data)

    def cleanup(self) -> None:
        self.ser.close()



from typing import Any
import yaml
from behave import given, then, when  # type: ignore
from mqtt_io.config import validate_and_normalise_main_config
from mqtt_io.exceptions import ConfigValidationFailed
from mqtt_io.types import ConfigType

# pylint: disable=function-redefined


@given("the config has an mqtt section")
def step(context: Any) -> None:
    config = context.data["raw_config"]
    config["mqtt"] = {}


@given("a valid config")  # type: ignore[no-redef]
def step(context: Any) -> None:
    config = context.data["raw_config"]
    config["mqtt"] = dict(host="localhost")


@given("the config has an entry in {section} with")  # type: ignore[no-redef]
def step(context: Any, section: str) -> None:
    data = yaml.safe_load(context.text)
    config = context.data["raw_config"]
    config.setdefault(section, []).append(data)


@given("the {section} config section dict contains")  # type: ignore[no-redef]
def step(context: Any, section: str) -> None:
    data = yaml.safe_load(context.text)
    config = context.data["raw_config"]
    config.setdefault(section, {}).update(data)


@when("we validate the main config")  # type: ignore[no-redef]
def step(context):
    try:
        context.data["config"] = validate_and_normalise_main_config(
            context.data["raw_config"]
        )
    except ConfigValidationFailed as exc:
        context.data["validation_error"] = exc


@then("config validation fails")  # type: ignore[no-redef]
def step(context):
    print("--- Config file ---")
    print(yaml.dump(context.data["raw_config"]))
    assert "validation_error" in context.data, "Config validation should have failed"


@then("the config validates")  # type: ignore[no-redef]
def step(context):
    print("--- Config file ---")
    print(yaml.dump(context.data["raw_config"]))
    try:
        print("--- Validation Errors ---")
        print(context.data["validation_error"])
    except KeyError:
        pass
    assert "validation_error" not in context.data, "Config should validate"
    assert (
        "config" in context.data
    ), 'Validated config should be in context.data["config"] (test error)'



"""
MCP3008 analog to digital converter
"""

import logging
from typing import cast

from mqtt_io.types import ConfigType, SensorValueType

from . import GenericSensor

REQUIREMENTS = ("adafruit-mcp3008",)

CONFIG_SCHEMA = {
    "spi_port": dict(type="integer", required=False, empty=False, default=0),
    "spi_device": dict(type="integer", required=False, empty=False, default=0),
    "chip_addr": dict(type="integer", required=False, empty=False, default=0),
}

_LOG = logging.getLogger("mqtt_gpio")


class Sensor(GenericSensor):
    """
    Implementation of MCP3008 ADC sensor.
    """

    SENSOR_SCHEMA = {
        "channel": dict(
            type="integer",
            required=True,
            min=0,
            max=7,
        )
    }

    def setup_module(self) -> None:
        """
        Init the mcp on SPI CE0
        """
        # pylint: disable=import-outside-toplevel,import-error
        import Adafruit_GPIO.SPI as SPI  # type: ignore
        import Adafruit_MCP3008  # type: ignore

        self.mcp = Adafruit_MCP3008.MCP3008(
            spi=SPI.SpiDev(self.config["spi_port"], self.config["spi_device"])
        )

    def get_value(self, sens_conf: ConfigType) -> SensorValueType:
        """
        Get the analog value from the adc for the configured channel
        """
        # Returns an integer from 0-1023
        return cast(int, self.mcp.read_adc(sens_conf["channel"]))
